// Game constants
const GRID_WIDTH = 10;
const GRID_HEIGHT = 20;
const BLOCK_SIZE = 30;
const CANVAS_WIDTH = GRID_WIDTH * BLOCK_SIZE;
const CANVAS_HEIGHT = GRID_HEIGHT * BLOCK_SIZE;

// Tetromino shapes
const SHAPES = [
  [[1, 1, 1, 1]],
  [[1, 1, 1, 1]],
  [[1, 1], [1, 1]],
  [[1, 1, 1], [0, 1, 0]],
  [[1, 1, 1], [1, 0, 0]],
  [[1, 1, 1], [0, 0, 1]],
  [[1, 1, 0], [0, 1, 1]],
  [[0, 1, 1], [1, 1, 0]]
];

// Colors for tetrominoes
const COLORS = [
  '#FF0000', '#00FF00', '#0000FF', '#FFFF00',
  '#00FFFF', '#FF00FF', '#FFA500', '#800080'
];

// Game variables
let canvas, ctx;
let grid;
let currentPiece, nextPiece;
let score = 0;
let gameOver = false;
let animationId;
let gamePaused = false; // Variable to track pause state
let dropSpeed = 200; // Normal drop speed

// Initialize the game
function init() {
  canvas = document.getElementById('gameCanvas');
  ctx = canvas.getContext('2d');
  canvas.width = CANVAS_WIDTH;
  canvas.height = CANVAS_HEIGHT;

  grid = Array(GRID_HEIGHT).fill().map(() => Array(GRID_WIDTH).fill(0));
  currentPiece = getRandomPiece();
  nextPiece = getRandomPiece();
  score = 0;
  gameOver = false;

  document.addEventListener('keydown', handleKeyPress);

  gameLoop();
}

// Game loop
function gameLoop() {
  if (!gamePaused) {
    update();
  }
  draw();
  
  animationId = setTimeout(() => {
    if (!gameOver) {
      gameLoop();
    }
  }, dropSpeed);
}

// Update game state
function update() {
  if (!moveDown()) {
    placePiece();
    clearLines();
    if (checkGameOver()) {
      gameOver = true;
    } else {
      currentPiece = nextPiece;
      nextPiece = getRandomPiece();
    }
  }
}

// Draw the game
function draw() {
  ctx.clearRect(0, 0, CANVAS_WIDTH, CANVAS_HEIGHT);
  drawGrid();
  drawPiece(currentPiece);
  drawScore();
  if (gameOver) {
    drawGameOver();
  }
}

// Draw the grid
function drawGrid() {
  for (let y = 0; y < GRID_HEIGHT; y++) {
    for (let x = 0; x < GRID_WIDTH; x++) {
      if (grid[y][x]) {
        ctx.fillStyle = COLORS[grid[y][x] - 1];
        ctx.fillRect(x * BLOCK_SIZE, y * BLOCK_SIZE, BLOCK_SIZE, BLOCK_SIZE);
        ctx.strokeStyle = '#000';
        ctx.strokeRect(x * BLOCK_SIZE, y * BLOCK_SIZE, BLOCK_SIZE, BLOCK_SIZE);
      }
    }
  }
}

// Draw the current piece
function drawPiece(piece) {
  ctx.fillStyle = piece.color; // Use piece's color directly
  piece.shape.forEach((row, y) => {
    row.forEach((value, x) => {
      if (value) {
        ctx.fillRect((piece.x + x) * BLOCK_SIZE, (piece.y + y) * BLOCK_SIZE, BLOCK_SIZE, BLOCK_SIZE);
        ctx.strokeStyle = '#000';
        ctx.strokeRect((piece.x + x) * BLOCK_SIZE, (piece.y + y) * BLOCK_SIZE, BLOCK_SIZE, BLOCK_SIZE);
      }
    });
  });
}

// Draw the score
function drawScore() {
  ctx.fillStyle = '#000';
  ctx.font = '20px Arial';
  ctx.fillText(`Score: ${score}`, 10, 30);
}

// Draw game over screen
function drawGameOver() {
  ctx.fillStyle = 'rgba(0, 0, 0, 0.75)';
  ctx.fillRect(0, 0, CANVAS_WIDTH, CANVAS_HEIGHT);
  ctx.fillStyle = '#FFF';
  ctx.font = '30px Arial';
  ctx.fillText('Game Over', CANVAS_WIDTH / 2 - 70, CANVAS_HEIGHT / 2 - 30);
  ctx.font = '20px Arial';
  ctx.fillText(`Final Score: ${score}`, CANVAS_WIDTH / 2 - 60, CANVAS_HEIGHT / 2 + 10);
  ctx.fillText('Press R to Restart', CANVAS_WIDTH / 2 - 80, CANVAS_HEIGHT / 2 + 50);
}

// Get a random piece
function getRandomPiece() {
  const shapeIndex = Math.floor(Math.random() * SHAPES.length); // random index for shape
  const shape = SHAPES[shapeIndex];
  return {
    shape: shape,
    color: COLORS[shapeIndex], // assign color to shape
    x: Math.floor(GRID_WIDTH / 2) - Math.floor(shape[0].length / 2),
    y: 0
  };
}

// Move the current piece down
function moveDown() {
  currentPiece.y++;
  if (hasCollision()) {
    currentPiece.y--;
    return false;
  }
  return true;
}

// Move the current piece left
function moveLeft() {
  currentPiece.x--;
  if (hasCollision()) {
    currentPiece.x++;
    return false;
  }
  return true;
}

// Move the current piece right
function moveRight() {
  currentPiece.x++;
  if (hasCollision()) {
    currentPiece.x--;
    return false;
  }
  return true;
}

// Rotate the current piece
function rotate() {
  const previousShape = currentPiece.shape;
  const rotated = currentPiece.shape[0].map((_, i) =>
    currentPiece.shape.map(row => row[i]).reverse()
  );
  
  currentPiece.shape = rotated;

  // Check for collision after rotation
  if (hasCollision()) {
    // If collision occurs, revert back to the original shape
    currentPiece.shape = previousShape;
    return false;
  }

  return true;
}

// Check for collisions
function hasCollision() {
  return currentPiece.shape.some((row, dy) =>
    row.some((value, dx) =>
      value &&
      (currentPiece.y + dy >= GRID_HEIGHT || 
       currentPiece.x + dx < 0 || 
       currentPiece.x + dx >= GRID_WIDTH || 
       grid[currentPiece.y + dy][currentPiece.x + dx])
    )
  );
}

// Place the current piece on the grid
function placePiece() {
  currentPiece.shape.forEach((row, y) => {
    row.forEach((value, x) => {
      if (value) {
        grid[currentPiece.y + y][currentPiece.x + x] = currentPiece.color; // Use the piece's color directly
      }
    });
  });
}

// Clear completed lines
function clearLines() {
  let linesCleared = 0;
  for (let y = GRID_HEIGHT - 1; y >= 0; y--) {
    if (grid[y].every(cell => cell !== 0)) {
      grid.splice(y, 1);
      grid.unshift(Array(GRID_WIDTH).fill(0));
      linesCleared++;
      y++;
    }
  }
  if (linesCleared > 0) {
    score += [40, 100, 300, 1200][linesCleared - 1];
  }
}

// Check if the game is over
function checkGameOver() {
  return grid[0].some(cell => cell !== 0);
}

// Handle key presses
function handleKeyPress(event) {
  if (event.code === 'Space') {
    gamePaused = !gamePaused; // Toggle pause state
    return;
  }

  if (gameOver) {
    if (event.key === 'r' || event.key === 'R') {
      cancelAnimationFrame(animationId);
      init();
    }
    return;
  }

 switch (event.key) {
    case 'ArrowLeft':
    case 'a': // Left movement with 'a'
        moveLeft();
        break;
    case 'ArrowRight':
    case 'd': // Right movement with 'd'
        moveRight();
        break;
    case 'ArrowDown':
    case 's': // Faster drop with 's'
        dropSpeed = 100; // Faster drop rate
        break;
    case 'ArrowUp':
    case 'w': // Rotate with 'w'
        rotate();
        break;
    case ' ':
        while (moveDown()) {} // Instant drop - already in place
        break;
}

}

// Reset drop speed when the down arrow is released
document.addEventListener('keyup', (event) => {
  if (event.key === 'ArrowDown' || event.key === 's') {
    dropSpeed = 200; // Reset drop rate to normal speed
  }
});

// Start the game
init();
